<a id="top"></a>

# Storyboard

<!-- toc -->
## Contents

  * [Why Use Storyboards](#why-use-storyboards)
  * [How To Use Storyboards](#how-to-use-storyboards)
    * [The Scenario](#the-scenario)
    * [Example Storyboard Code](#example-storyboard-code)
    * [Making Objects 'Storyboard Friendly'](#making-objects-storyboard-friendly)<!-- endToc -->

## Why Use Storyboards

Sometimes we might want to see different steps in a workflow or lifetime of an object. Storyboards are a convenience object to help enable that.

ApprovalTests allows us to look at a complete object instead of just pieces of it. Storyboards allow us to track an object through time.

The mechanism to map time to space that storyboards use is very analogous to a comic book, but with each frame vertically after each other so that it works well with the diff tool and shows a progression.


## How To Use Storyboards

Here's a full example using most of the Storyboard features. 

### The Scenario

For context, this is the `.approved.` file generated by the source code below:

<!-- snippet: StoryboardTests.Storyboard_customization_mechanisms.approved.txt -->
<a id='snippet-StoryboardTests.Storyboard_customization_mechanisms.approved.txt'></a>
```txt
Game of Life

Initial Frame:
. . . . . 
. . . . . 
. X X X . 
. . . . . 
. . . . . 


Start game:
. . . . . 
. . X . . 
. . X . . 
. . X . . 
. . . . . 


Frame #2:
. . . . . 
. . . . . 
. X X X . 
. . . . . 
. . . . . 


setting alive: *
setting dead: _

Frame #3:
_ _ _ _ _ 
_ _ * _ _ 
_ _ * _ _ 
_ _ * _ _ 
_ _ _ _ _ 


Frame #4:
_ _ _ _ _ 
_ _ _ _ _ 
_ * * * _ 
_ _ _ _ _ 
_ _ _ _ _ 


Frame #5:
_ _ _ _ _ 
_ _ * _ _ 
_ _ * _ _ 
_ _ * _ _ 
_ _ _ _ _ 



```
<sup><a href='/tests/DocTest_Tests/approval_tests/StoryboardTests.Storyboard_customization_mechanisms.approved.txt#L1-L54' title='Snippet source file'>snippet source</a> | <a href='#snippet-StoryboardTests.Storyboard_customization_mechanisms.approved.txt' title='Start of snippet'>anchor</a></sup>
<!-- endSnippet -->

### Example Storyboard Code

which is produced from the following code:

<!-- snippet: storyboard_complete_example -->
<a id='snippet-storyboard_complete_example'></a>
```cpp
// Create a Storyboard to track a series of changes
ApprovalTests::Storyboard story;

// Create object to track
GameOfLife game = createBlinker();

// Add a title
story.addDescription("Game of Life");

// Capture the starting state
story.addFrame(game.toString());

// Frame1: with title
game = game.advance();
story.addFrame("Start game", game.toString());

// Frame2: default titling
game = game.advance();
story.addFrame(game.toString());

// Change how it renders and note it in the Storyboard
story.addDescriptionWithData("setting alive", game.setAliveCell("*"));
story.addDescriptionWithData("setting dead", game.setDeadCell("_"));

// 3 more frames
for (int i = 0; i < 3; ++i)
{
    game = game.advance();
    story.addFrame(game.toString());
}

// verify storyboard
ApprovalTests::Approvals::verify(story);
```
<sup><a href='/tests/DocTest_Tests/StoryboardTests.cpp#L61-L95' title='Snippet source file'>snippet source</a> | <a href='#snippet-storyboard_complete_example' title='Start of snippet'>anchor</a></sup>
<!-- endSnippet -->

### Making Objects 'Storyboard Friendly'

A technique commonly used with storyboards is to make functions that would often return 'void' return objects that show the changes.


For example:
The following is a setter, which would normally return void. Here we have it return the value set, so we can easily capture it in the Storyboard.

<!-- snippet: storyboard_return_values -->
<a id='snippet-storyboard_return_values'></a>
```h
std::string setAliveCell(std::string alive)
{
    aliveCharacter = std::move(alive);
    return aliveCharacter;
}
```
<sup><a href='/tests/DocTest_Tests/GameOfLife.h#L178-L184' title='Snippet source file'>snippet source</a> | <a href='#snippet-storyboard_return_values' title='Start of snippet'>anchor</a></sup>
<!-- endSnippet -->

This means we can write:
<!-- snippet: storyboard_friendly_report -->
<a id='snippet-storyboard_friendly_report'></a>
```cpp
story.addDescriptionWithData("setting alive", game.setAliveCell("*"));
```
<sup><a href='/tests/DocTest_Tests/StoryboardTests.cpp#L48-L50' title='Snippet source file'>snippet source</a> | <a href='#snippet-storyboard_friendly_report' title='Start of snippet'>anchor</a></sup>
<!-- endSnippet -->

instead of
<!-- snippet: storyboard_unfriendly_report -->
<a id='snippet-storyboard_unfriendly_report'></a>
```cpp
std::string newValue = "*";
game.setAliveCell(newValue);
story.addDescriptionWithData("setting alive", newValue);
```
<sup><a href='/tests/DocTest_Tests/StoryboardTests.cpp#L52-L56' title='Snippet source file'>snippet source</a> | <a href='#snippet-storyboard_unfriendly_report' title='Start of snippet'>anchor</a></sup>
<!-- endSnippet -->

---

[Back to User Guide](/doc/README.md#top)
